(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(global = global || self, factory(global.webrtcRemoteControl = {}));
})(this, (function (exports) {
	function createCommonjsModule(fn) {
	  var module = { exports: {} };
		return fn(module, module.exports), module.exports;
	}

	var eventemitter3 = createCommonjsModule(function (module) {

	  var has = Object.prototype.hasOwnProperty,
	      prefix = '~';
	  /**
	   * Constructor to create a storage for our `EE` objects.
	   * An `Events` instance is a plain object whose properties are event names.
	   *
	   * @constructor
	   * @private
	   */

	  function Events() {} //
	  // We try to not inherit from `Object.prototype`. In some engines creating an
	  // instance in this way is faster than calling `Object.create(null)` directly.
	  // If `Object.create(null)` is not supported we prefix the event names with a
	  // character to make sure that the built-in object properties are not
	  // overridden or used as an attack vector.
	  //


	  if (Object.create) {
	    Events.prototype = Object.create(null); //
	    // This hack is needed because the `__proto__` property is still inherited in
	    // some old browsers like Android 4, iPhone 5.1, Opera 11 and Safari 5.
	    //

	    if (!new Events().__proto__) prefix = false;
	  }
	  /**
	   * Representation of a single event listener.
	   *
	   * @param {Function} fn The listener function.
	   * @param {*} context The context to invoke the listener with.
	   * @param {Boolean} [once=false] Specify if the listener is a one-time listener.
	   * @constructor
	   * @private
	   */


	  function EE(fn, context, once) {
	    this.fn = fn;
	    this.context = context;
	    this.once = once || false;
	  }
	  /**
	   * Add a listener for a given event.
	   *
	   * @param {EventEmitter} emitter Reference to the `EventEmitter` instance.
	   * @param {(String|Symbol)} event The event name.
	   * @param {Function} fn The listener function.
	   * @param {*} context The context to invoke the listener with.
	   * @param {Boolean} once Specify if the listener is a one-time listener.
	   * @returns {EventEmitter}
	   * @private
	   */


	  function addListener(emitter, event, fn, context, once) {
	    if (typeof fn !== 'function') {
	      throw new TypeError('The listener must be a function');
	    }

	    var listener = new EE(fn, context || emitter, once),
	        evt = prefix ? prefix + event : event;
	    if (!emitter._events[evt]) emitter._events[evt] = listener, emitter._eventsCount++;else if (!emitter._events[evt].fn) emitter._events[evt].push(listener);else emitter._events[evt] = [emitter._events[evt], listener];
	    return emitter;
	  }
	  /**
	   * Clear event by name.
	   *
	   * @param {EventEmitter} emitter Reference to the `EventEmitter` instance.
	   * @param {(String|Symbol)} evt The Event name.
	   * @private
	   */


	  function clearEvent(emitter, evt) {
	    if (--emitter._eventsCount === 0) emitter._events = new Events();else delete emitter._events[evt];
	  }
	  /**
	   * Minimal `EventEmitter` interface that is molded against the Node.js
	   * `EventEmitter` interface.
	   *
	   * @constructor
	   * @public
	   */


	  function EventEmitter() {
	    this._events = new Events();
	    this._eventsCount = 0;
	  }
	  /**
	   * Return an array listing the events for which the emitter has registered
	   * listeners.
	   *
	   * @returns {Array}
	   * @public
	   */


	  EventEmitter.prototype.eventNames = function eventNames() {
	    var names = [],
	        events,
	        name;
	    if (this._eventsCount === 0) return names;

	    for (name in events = this._events) {
	      if (has.call(events, name)) names.push(prefix ? name.slice(1) : name);
	    }

	    if (Object.getOwnPropertySymbols) {
	      return names.concat(Object.getOwnPropertySymbols(events));
	    }

	    return names;
	  };
	  /**
	   * Return the listeners registered for a given event.
	   *
	   * @param {(String|Symbol)} event The event name.
	   * @returns {Array} The registered listeners.
	   * @public
	   */


	  EventEmitter.prototype.listeners = function listeners(event) {
	    var evt = prefix ? prefix + event : event,
	        handlers = this._events[evt];
	    if (!handlers) return [];
	    if (handlers.fn) return [handlers.fn];

	    for (var i = 0, l = handlers.length, ee = new Array(l); i < l; i++) {
	      ee[i] = handlers[i].fn;
	    }

	    return ee;
	  };
	  /**
	   * Return the number of listeners listening to a given event.
	   *
	   * @param {(String|Symbol)} event The event name.
	   * @returns {Number} The number of listeners.
	   * @public
	   */


	  EventEmitter.prototype.listenerCount = function listenerCount(event) {
	    var evt = prefix ? prefix + event : event,
	        listeners = this._events[evt];
	    if (!listeners) return 0;
	    if (listeners.fn) return 1;
	    return listeners.length;
	  };
	  /**
	   * Calls each of the listeners registered for a given event.
	   *
	   * @param {(String|Symbol)} event The event name.
	   * @returns {Boolean} `true` if the event had listeners, else `false`.
	   * @public
	   */


	  EventEmitter.prototype.emit = function emit(event, a1, a2, a3, a4, a5) {
	    var evt = prefix ? prefix + event : event;
	    if (!this._events[evt]) return false;
	    var listeners = this._events[evt],
	        len = arguments.length,
	        args,
	        i;

	    if (listeners.fn) {
	      if (listeners.once) this.removeListener(event, listeners.fn, undefined, true);

	      switch (len) {
	        case 1:
	          return listeners.fn.call(listeners.context), true;

	        case 2:
	          return listeners.fn.call(listeners.context, a1), true;

	        case 3:
	          return listeners.fn.call(listeners.context, a1, a2), true;

	        case 4:
	          return listeners.fn.call(listeners.context, a1, a2, a3), true;

	        case 5:
	          return listeners.fn.call(listeners.context, a1, a2, a3, a4), true;

	        case 6:
	          return listeners.fn.call(listeners.context, a1, a2, a3, a4, a5), true;
	      }

	      for (i = 1, args = new Array(len - 1); i < len; i++) {
	        args[i - 1] = arguments[i];
	      }

	      listeners.fn.apply(listeners.context, args);
	    } else {
	      var length = listeners.length,
	          j;

	      for (i = 0; i < length; i++) {
	        if (listeners[i].once) this.removeListener(event, listeners[i].fn, undefined, true);

	        switch (len) {
	          case 1:
	            listeners[i].fn.call(listeners[i].context);
	            break;

	          case 2:
	            listeners[i].fn.call(listeners[i].context, a1);
	            break;

	          case 3:
	            listeners[i].fn.call(listeners[i].context, a1, a2);
	            break;

	          case 4:
	            listeners[i].fn.call(listeners[i].context, a1, a2, a3);
	            break;

	          default:
	            if (!args) for (j = 1, args = new Array(len - 1); j < len; j++) {
	              args[j - 1] = arguments[j];
	            }
	            listeners[i].fn.apply(listeners[i].context, args);
	        }
	      }
	    }

	    return true;
	  };
	  /**
	   * Add a listener for a given event.
	   *
	   * @param {(String|Symbol)} event The event name.
	   * @param {Function} fn The listener function.
	   * @param {*} [context=this] The context to invoke the listener with.
	   * @returns {EventEmitter} `this`.
	   * @public
	   */


	  EventEmitter.prototype.on = function on(event, fn, context) {
	    return addListener(this, event, fn, context, false);
	  };
	  /**
	   * Add a one-time listener for a given event.
	   *
	   * @param {(String|Symbol)} event The event name.
	   * @param {Function} fn The listener function.
	   * @param {*} [context=this] The context to invoke the listener with.
	   * @returns {EventEmitter} `this`.
	   * @public
	   */


	  EventEmitter.prototype.once = function once(event, fn, context) {
	    return addListener(this, event, fn, context, true);
	  };
	  /**
	   * Remove the listeners of a given event.
	   *
	   * @param {(String|Symbol)} event The event name.
	   * @param {Function} fn Only remove the listeners that match this function.
	   * @param {*} context Only remove the listeners that have this context.
	   * @param {Boolean} once Only remove one-time listeners.
	   * @returns {EventEmitter} `this`.
	   * @public
	   */


	  EventEmitter.prototype.removeListener = function removeListener(event, fn, context, once) {
	    var evt = prefix ? prefix + event : event;
	    if (!this._events[evt]) return this;

	    if (!fn) {
	      clearEvent(this, evt);
	      return this;
	    }

	    var listeners = this._events[evt];

	    if (listeners.fn) {
	      if (listeners.fn === fn && (!once || listeners.once) && (!context || listeners.context === context)) {
	        clearEvent(this, evt);
	      }
	    } else {
	      for (var i = 0, events = [], length = listeners.length; i < length; i++) {
	        if (listeners[i].fn !== fn || once && !listeners[i].once || context && listeners[i].context !== context) {
	          events.push(listeners[i]);
	        }
	      } //
	      // Reset the array, or remove it completely if we have no more listeners.
	      //


	      if (events.length) this._events[evt] = events.length === 1 ? events[0] : events;else clearEvent(this, evt);
	    }

	    return this;
	  };
	  /**
	   * Remove all listeners, or those of the specified event.
	   *
	   * @param {(String|Symbol)} [event] The event name.
	   * @returns {EventEmitter} `this`.
	   * @public
	   */


	  EventEmitter.prototype.removeAllListeners = function removeAllListeners(event) {
	    var evt;

	    if (event) {
	      evt = prefix ? prefix + event : event;
	      if (this._events[evt]) clearEvent(this, evt);
	    } else {
	      this._events = new Events();
	      this._eventsCount = 0;
	    }

	    return this;
	  }; //
	  // Alias methods names because people roll like that.
	  //


	  EventEmitter.prototype.off = EventEmitter.prototype.removeListener;
	  EventEmitter.prototype.addListener = EventEmitter.prototype.on; //
	  // Expose the prefix.
	  //

	  EventEmitter.prefixed = prefix; //
	  // Allow `EventEmitter` to be imported as module namespace.
	  //

	  EventEmitter.EventEmitter = EventEmitter; //
	  // Expose the module.
	  //

	  {
	    module.exports = EventEmitter;
	  }
	});

	function makeStoreAccessor(sessionStorageKey = "webrtc-remote-control-peer-id") {
	  return {
	    getPeerId() {
	      return sessionStorage.getItem(sessionStorageKey);
	    },

	    setPeerIdToSessionStorage(peerId) {
	      sessionStorage.setItem(sessionStorageKey, peerId);
	    }

	  };
	}
	function makeConnectionFilterUtilities() {
	  const connMetadata = "from-webrtc-remote-control";
	  return {
	    isConnectionFromRemote(conn) {
	      return conn.metadata === connMetadata;
	    },

	    connMetadata
	  };
	}
	/**
	 * Pass mapping of error.type / message
	 * You can pass `default` key a function that accepts an `error` and returns a string
	 */

	function makeHumanizeError({
	  mapping: overrideMapping,
	  withTechicalErrorMessage
	} = {
	  mapping: {},
	  withTechicalErrorMessage: true
	}) {
	  const mapping = {
	    "browser-incompatible": "Your browser doesn't support WebRTC features, please try with a recent browser.",
	    disconnected: "You are disconnected and can't make any more peer connection, please reload.",
	    network: "It seems you're experimenting some network problems.",
	    "peer-unavailable": "The peer you were connected to seems to have lost connection, try to reload it.",
	    "server-error": "An error occured on the signaling server. Sorry, try to come back later.",
	    default: error => `An error occured${error.type ? ` - type: ${error.type}` : ""}`,
	    ...overrideMapping
	  };
	  return function humanizeError(error) {
	    const humanError = mapping[error.type] || (typeof mapping.default === "function" ? mapping.default(error) : mapping.default);
	    return humanError && error.message && withTechicalErrorMessage ? `${humanError} (${error.message})` : humanError;
	  };
	}
	function prepareUtils({
	  sessionStorageKey,
	  humanErrors
	} = {}) {
	  const humanizeError = makeHumanizeError(humanErrors);
	  const {
	    isConnectionFromRemote
	  } = makeConnectionFilterUtilities();
	  const {
	    getPeerId,
	    setPeerIdToSessionStorage
	  } = makeStoreAccessor(sessionStorageKey);
	  return {
	    humanizeError,
	    isConnectionFromRemote,
	    getPeerId,
	    setPeerIdToSessionStorage
	  };
	}

	/* eslint-disable import/no-relative-packages,import/no-extraneous-dependencies */
	function prepare$1({
	  humanizeError,
	  isConnectionFromRemote,
	  getPeerId,
	  setPeerIdToSessionStorage
	}) {
	  return {
	    humanizeError,
	    isConnectionFromRemote,
	    getPeerId,

	    bindConnection(peer) {
	      return new Promise(res => {
	        const ee = new eventemitter3();
	        const connections = new Map();
	        const wrcMaster = {
	          sendTo(id, payload) {
	            const conn = connections.get(id);

	            if (conn) {
	              return conn.send(payload);
	            }

	            return null;
	          },

	          sendAll(payload) {
	            [...connections.values()].forEach(conn => {
	              conn.send(payload);
	            });
	          },

	          on: ee.on.bind(ee),
	          off: ee.off.bind(ee)
	        };
	        peer.on("open", peerId => {
	          setPeerIdToSessionStorage(peerId);
	          res(wrcMaster);
	        });
	        peer.on("connection", conn => {
	          // we don't track the connections made by the user directly using `peer.connect`
	          if (!isConnectionFromRemote(conn)) {
	            return;
	          } // if this is a reconnect from the same peer, replace connection with the latest one


	          connections.set(conn.peer, conn);
	          conn.on("open", () => {
	            ee.emit("remote.connect", {
	              id: conn.peer
	            });
	            console.log("connections", connections);
	          });
	          conn.on("data", data => {
	            ee.emit("data", {
	              id: conn.peer,
	              from: "remote"
	            }, data);
	          });
	          conn.on("close", () => {
	            connections.delete(conn.peer);
	            ee.emit("remote.disconnect", {
	              id: conn.peer
	            });
	            console.log("connections", connections);
	          });
	        });
	      });
	    }

	  };
	}

	var core_master = {
		__proto__: null,
		'default': prepare$1,
		prepareUtils: prepareUtils
	};

	/* eslint-disable import/no-relative-packages,import/no-extraneous-dependencies */

	function makePeerConnection(peer, masterPeerId, ee, onConnectionOpened) {
	  const {
	    connMetadata
	  } = makeConnectionFilterUtilities(); // to ensure connections with iOs, must use json serialization

	  const conn = peer.connect(masterPeerId, {
	    serialization: "json",
	    metadata: connMetadata // will let us identify which connections are managed by the package / by the user

	  });
	  conn.on("open", () => {
	    if (typeof onConnectionOpened === "function") {
	      onConnectionOpened();
	    }
	  });
	  conn.on("data", data => {
	    ee.emit("data", {
	      from: "master"
	    }, data);
	  });
	  return conn;
	}

	function prepare({
	  humanizeError,
	  getPeerId,
	  setPeerIdToSessionStorage
	}) {
	  return {
	    humanizeError,
	    getPeerId,

	    bindConnection(peer, masterPeerId) {
	      return new Promise(res => {
	        let conn = null;
	        const ee = new eventemitter3();
	        const wrcRemote = {
	          send(payload) {
	            if (conn) {
	              conn.send(payload);
	            } else {
	              // eslint-disable-next-line no-console
	              console.warning("You called `send` with no connection");
	            }
	          },

	          on: ee.on.bind(ee),
	          off: ee.off.bind(ee)
	        };

	        const createPeerConnectionWithReconnectOnClose = onConnectionOpened => {
	          conn = null;
	          conn = makePeerConnection(peer, masterPeerId, ee, onConnectionOpened);
	          conn.on("close", () => {
	            ee.emit("remote.disconnect", {
	              id: peer.id
	            });
	            createPeerConnectionWithReconnectOnClose(() => {
	              ee.emit("remote.reconnect", {
	                id: peer.id
	              });
	            });
	          });
	        };

	        peer.on("open", peerId => {
	          setPeerIdToSessionStorage(peerId);
	          createPeerConnectionWithReconnectOnClose(() => res(wrcRemote));
	          conn.on("error", e => {
	            // todo emit some error ? same on master ?
	            console.log("conn.error", e);
	          }); // ensure to disconnect remote when the page is closed

	          const onBeforeUnloadPeerDisconnect = () => {
	            if (conn && conn.disconnect) {
	              conn.disconnect();
	            }
	          };

	          window.addEventListener("beforeunload", onBeforeUnloadPeerDisconnect);
	        });
	      });
	    }

	  };
	}

	var core_remote = {
		__proto__: null,
		'default': prepare,
		prepareUtils: prepareUtils
	};

	exports.master = core_master;
	exports.prepareUtils = prepareUtils;
	exports.remote = core_remote;

}));
//# sourceMappingURL=webrtc-remote-control.umd.dev.js.map
